//
//  STTwitterStreamer.m
//  STTwitterStreamer
//
//  Created by Sebastien Thiebaud on 1/19/13.
//  Copyright (c) 2013 Sebastien Thiebaud. All rights reserved.
//

#import "STTwitterStreamer.h"

@interface STTwitterStreamer()

- (void)initialize;

@end

@implementation STTwitterStreamer

#pragma mark -
#pragma mark Singleton

+ (STTwitterStreamer *)stream
{    
    static STTwitterStreamer *_instance;

    @synchronized(self)
    {
        if (_instance == nil)
        {
            _instance = [[STTwitterStreamer alloc] init];
        }
    }
    
    return _instance;
}



- (id)init
{
    if (self = [super init])
    {
        _tweets = [[NSMutableArray alloc] init];
    }

    return self;
}

- (void)setDelegate:(id<STTwitterStreamerDelegate>)delegate
{
    _delegate = delegate;
    
    [self initialize];
}

- (void)initialize
{
    if (_account == nil)
    {
        _accountStore = [[ACAccountStore alloc] init];
        ACAccountType *accountType = [_accountStore accountTypeWithAccountTypeIdentifier:ACAccountTypeIdentifierTwitter];
        
        [_accountStore requestAccessToAccountsWithType:accountType options:nil completion:^(BOOL granted, NSError *error) {
            if (granted == YES)
            {
                NSArray *accounts = [_accountStore accountsWithAccountType:accountType];
                
                if ([accounts count] == 0)
                {
                    if ([_delegate respondsToSelector:@selector(noAccountConfigured)])
                    {
                        [_delegate noAccountConfigured];
                    }
                }
                else if ([accounts count] == 1)
                {
                    _account = [accounts lastObject];
                    [_delegate streamAvailable];
                }
                else if ([accounts count] > 1)
                {
                    [_delegate multipleAccountsConfigured:accounts];
                }
            }
        }];
    }
}

- (void)settingUpWithAccount:(ACAccount *)account
{
    _account = account;
    [_delegate streamAvailable];
}

- (void)start
{
    if (_account != nil)
    {
        NSURL *url = [NSURL URLWithString:@"https://userstream.twitter.com/1.1/user.json"];
        
        NSMutableDictionary *params = [[NSMutableDictionary alloc] init];
        [params setObject:@"length" forKey:@"delimited"];

        SLRequest *request = [SLRequest requestForServiceType:SLServiceTypeTwitter requestMethod:SLRequestMethodGET URL:url parameters:params];
        [request setAccount:_account];
        
        _connection = [[NSURLConnection alloc] initWithRequest:[request preparedURLRequest] delegate:self startImmediately:YES];
    }
    else
    {
        [_delegate noAccountConfigured];
    }
}

- (void)stop
{
    [_connection cancel];
    
    if ([_delegate respondsToSelector:@selector(streamStopped)])
    {
        [_delegate streamStopped];
    }
}

#pragma mark -
#pragma mark NSURLConnection Data Delegate

- (void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error
{
    NSLog(@"%@", error);
}

- (void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)data
{
    NSArray *explode = [[[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding] componentsSeparatedByCharactersInSet:[NSCharacterSet newlineCharacterSet]];
    NSString *stringJSON = [explode objectAtIndex:2];
    NSDictionary *JSON = [NSJSONSerialization JSONObjectWithData:[stringJSON dataUsingEncoding:NSUTF8StringEncoding] options:NSJSONReadingMutableContainers error:nil];
    
    if ([JSON objectForKey:@"friends"] != nil)
    {
        _friends = [JSON objectForKey:@"friends"];
    }
    else if ([JSON objectForKey:@"delete"] != nil)
    {
        NSPredicate *predicate = [NSPredicate predicateWithFormat:@"id_str = %@", [[[JSON objectForKey:@"delete"] objectForKey:@"status"] objectForKey:@"id_str"]];
        
        NSArray *result = [_tweets filteredArrayUsingPredicate:predicate];
        
        if ([result count] == 1)
        {
            int index = [_tweets indexOfObject:[result objectAtIndex:0]];
            [_tweets removeObjectAtIndex:index];
            [_delegate deleteTweet:[result objectAtIndex:0] atIndex:index];
        }
    }
    else if (JSON != nil)
    {
        [_tweets addObject:JSON];
        [_delegate newTweet:JSON];
    }
}

@end
