//
//  STTwitterViewController.m
//  STTwitterStreamerExample
//
//  Created by Sebastien Thiebaud on 1/20/13.
//  Copyright (c) 2013 Sebastien Thiebaud. All rights reserved.
//

#import "STTwitterViewController.h"

@interface STTwitterViewController ()

@end

@implementation STTwitterViewController

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
    }
    return self;
}

- (void)viewDidLoad
{
    [super viewDidLoad];

    [_button setTitle:@"Wait..." forState:UIControlStateNormal];
    [_button setEnabled:NO];
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
    
    [[STTwitterStreamer stream] setDelegate:self];
}

#pragma mark -
#pragma mark Event methods

- (IBAction)startStreaming:(UIButton *)sender
{
    [[STTwitterStreamer stream] start];
    
    [_button setTitle:@"In progress..." forState:UIControlStateNormal];
    [_button setEnabled:NO];
}

#pragma mark -
#pragma mark STTwitterStream Delegate

- (void)multipleAccountsConfigured:(NSArray *)accounts
{
    _tempAccounts = accounts;
    
    UIActionSheet *actionSheet = [[UIActionSheet alloc] initWithTitle:@"Twitter accounts" delegate:self cancelButtonTitle:nil destructiveButtonTitle:nil otherButtonTitles:nil];
    
    for (ACAccount *account in accounts)
    {
        [actionSheet addButtonWithTitle:account.username];
    }
    
    [actionSheet showInView:self.view];
}

- (void)noAccountConfigured
{
    UIAlertView *alertView = [[UIAlertView alloc] initWithTitle:@"No Twitter account" message:@"Please, set up a Twitter account to access on the app." delegate:nil cancelButtonTitle:nil otherButtonTitles:@"OK", nil];
    [alertView show];
}

- (void)streamAvailable
{
    [_button setTitle:@"Start streaming" forState:UIControlStateNormal];
    [_button setEnabled:YES];
}

- (void)streamStopped
{
    [_button setTitle:@"Streaming stopped" forState:UIControlStateNormal];
    [_button setEnabled:YES];
}

- (void)newTweet:(NSDictionary *)informations
{
    [_tableView beginUpdates];
    [_tableView insertRowsAtIndexPaths:[NSArray arrayWithObject:[NSIndexPath indexPathForRow:0 inSection:0]] withRowAnimation:UITableViewRowAnimationTop];
    [_tableView endUpdates];
}

- (void)deleteTweet:(NSDictionary *)informations atIndex:(int)index
{
    int indexToDelete = [[[STTwitterStreamer stream] tweets] count] - index;
    indexToDelete = (indexToDelete < 0) ? 0 : indexToDelete;
    
    [_tableView beginUpdates];
    [_tableView deleteRowsAtIndexPaths:[NSArray arrayWithObject:[NSIndexPath indexPathForRow:indexToDelete inSection:0]] withRowAnimation:UITableViewRowAnimationRight];
    [_tableView endUpdates];
}

#pragma mark -
#pragma mark UIActionSheet Delegate

- (void)actionSheet:(UIActionSheet *)actionSheet clickedButtonAtIndex:(NSInteger)buttonIndex
{
    [[STTwitterStreamer stream] settingUpWithAccount:[_tempAccounts objectAtIndex:buttonIndex]];
}

#pragma mark -
#pragma mark UITableView Datasource

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    return [[[STTwitterStreamer stream] tweets] count];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString *cellIdentifier = @"tweetCell";
    
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:cellIdentifier];
    
    if (cell == nil)
    {
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:cellIdentifier];        
    }
        
    cell.textLabel.text = [[[[STTwitterStreamer stream] tweets] objectAtIndex:[[[STTwitterStreamer stream] tweets] count] - indexPath.row - 1] objectForKey:@"text"];

    return cell;
}

@end
